#!/bin/bash
# latest-brave Version 0.1
# based on latest-chrome Version 1.3.3 by Ruari
# hacked for puppy by 01micko

# This script will find the latest Google Chrome binary package,
# download it and repackage it into Slackware format.

# I don't use Chrome for regular browsing but it is handy for
# comparative tests against Vivaldi. :P

# Copyright 2016 Ruari Oedegaard, Oslo, Norway
# All rights reserved.
#
# Redistribution and use of this script, with or without modification, is
# permitted provided that the following conditions are met:
#
# 1. Redistributions of this script must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


# Use the architecture of the current machine or whatever the user has
# set externally
ARCH=${ARCH:-$(uname -m)}
if [ ! "$ARCH" = "x86_64" ]; then
  echo "The architecture $ARCH is not supported." >&2
  exit 1
fi

[ "$1" ] && TAG=$1 || TAG=~spot

# get the brave URL
BRAVEURL=${BRAVEURL:-$(curl -s https://slackbuilds.org/repository/14.2/network/brave-browser/ | grep "Source Downloads" | grep -o 'https.*rpm">' | sed 's%\">%%')}

# Work out the latest stable Brave if VERSION is unset
VERSION=${VERSION:-$(echo $BRAVEURL | grep -o 'browser\-.*\.x86' | sed -e 's%\([a-z]*[a-z]\-\)%%' -e 's%\.[a-z].*$%%')}

# Error out if $VERISON is unset, e.g. because previous command failed
if [ -z $VERSION ]; then
  echo "Could not work out the latest version; exiting" >&2
  exit 1
fi

if echo $VERSION | grep -Fq '-'; then
  BRAVEREVISION=${VERSION#*-}
  VERSION=${VERSION%-*}
else
  BRAVEREVISION=""
fi
echo "new: $VERSION"

CWD=$(pwd)
TMP=${TMP:-/tmp}
REPACKDIR=$TMP/repackage-brave

# Since packaging is about to begin errors become more important now,
# so exit if things fail.
set -eu

# If the repackage is already present from the past, clear it down
# and re-create it.
if [ -d "$REPACKDIR" ]; then
  rm -fr "$REPACKDIR"
fi

mkdir -p "$REPACKDIR"/pkg/src
mkdir -p "$REPACKDIR"/src

# Check if the current directory contains the correct Google Chrome
# binary package, otherwise download it. Then check that it matches the
# version number defined.
if [ -e brave-browser-${VERSION}-${BRAVEREVISION}.${ARCH}.rpm ]; then
  (
    cd "$REPACKDIR/src/"
    ln -s "$CWD/brave-browser-${VERSION}-${BRAVEREVISION}.${ARCH}.rpm" brave-browser-${VERSION}-${BRAVEREVISION}.${ARCH}.rpm
  )
else
  echo "Downloading Brave ${VERSION} for ${ARCH}"
  wget "$BRAVEURL" -P "$REPACKDIR/src/"
fi

# Now we have all the sources in place, switch to the package directory
# and start setting things up.
cd "$REPACKDIR/pkg"

# Extract the contents of the Bravee binary package
if [ -x /usr/bin/bsdtar ]; then
  bsdtar xf ../src/brave*${ARCH}.rpm
elif [ -x /usr/bin/rpm2cpio ]; then
  rpm2cpio ../src/brave*${ARCH}.rpm | cpio --quiet -id
else
  # Since the user has not installed libarchive or rpm, use a hack to extract the rpm contents
  RPMHDRLGTH=$(LANG=C grep -abom1 '.7zXZ\|BZh9' ../src/brave*${ARCH}.rpm)
  case "$RPMHDRLGTH" in
    *7zXZ) COMPRESSOR=xz ;;
    *BZh9) COMPRESSOR=bzip2 ;;
    *)     echo "Unknown compression type in rpm!" >&2; exit 1 ;;
  esac
  tail -c+$[${RPMHDRLGTH%:*}+1] ../src/brave*${ARCH}.rpm | $COMPRESSOR -d | cpio --quiet -id
fi

# Remove ./etc as it contains a cron job for updating rpm systems
rm -fr etc

# Replace brave executable symlink, with a relative one
if [ -h usr/bin/brave-browser-stable ]; then
  rm usr/bin/brave-browser-stable
  (
    cd usr/bin
    ln -fs ../../opt/brave.com/brave/brave-browser brave-browser-stable
  )
fi


# Symlink desktop environment icons
for png in opt/brave.com/brave/product_logo_*.png; do
  pngsize="${png##*/product_logo_}"
  mkdir -p usr/share/icons/hicolor/${pngsize%.png}x${pngsize%.png}/apps
  (
    cd usr/share/icons/hicolor/${pngsize%.png}x${pngsize%.png}/apps/
    ln -s ../../../../../../opt/brave.com/brave/product_logo_${pngsize} brave-browser.png
  )
done

# fix .desktop
case $TAG in
	*spot)
	sed -i -e 's/Icon=.*/Icon=brave.svg/' \
		-e "s/Exec=/Exec=run-as-spot /" \
		-e 's/ %U//' \
		-e 's/Categories=.*/Categories=WebBrowser;/' \
		usr/share/applications/*.desktop
		;;
	*root)
	sed -i -e 's/Icon=.*/Icon=brave.svg/' \
		-e "s/Exec=\/usr\/bin\/brave-browser-stable/Exec=\/usr\/bin\/brave-browser-stable --no-sandbox /" \
		-e 's/ %U//' \
		-e 's/Categories=.*/Categories=WebBrowser;/' \
		usr/share/applications/*.desktop
		;;
esac	

# add brave.svg icon
mkdir -p usr/share/pixmaps
cat > usr/share/pixmaps/brave.svg << EOF
<?xml version="1.0" encoding="UTF-8"?>
<svg xmlns="http://www.w3.org/2000/svg" version="1.1" height="48px" width="48px" viewBox="0 0 192 192">
  <path style="fill:#F76F31;stroke:none;stroke-width:1;" d="m 66,14 60,0 14,16 10,-2 10,10 0,10 14,20 -18,70 a 20 20 0 0 1 -8,10 l -40,30 a 20 20 0 0 1 -24,0 l -40,-30 a 20 20 0 0 1 -8,-10 l -18,-70 14,-20 0,-10 10,-10 10,2 z" />
  <path style="fill:#FFF;stroke:none;" d="m 56,42 14,4 a 20 20 0 0 0 8,0 l 14,-4 a 20,20 0 0 1 8,0 l 14,4 a 20 20 0 0 0 8,0 l 14,-4 20,28 a 10 10 0 0 1 0,8 l -22,22 3,8 a 20 15 0 0 1 0,8 a 25 25 0 0 1 -8,8 a 8 8 0 0 1 -6,0 a 40 40 0 0 1 -20,-14 a 4 4 0 0 1 0,-4 l 12,-12 a 4 4 0 0 0 0,-4 l -4,-12 a 10 10 0 0 1 2,-7 a 40 40 0 0 1 24,-10 a 60 60 0 0 0 -30,6 l 4,24 a 40,40 0 0 1 -30,0 l 4,-24 a 60 60 0 0 0 -30,-6 a 40 40 0 0 1 24,10 a 10 10 0 0 1 2,7 l -4,12 a 4 4 0 0 0 0,4 l 12,12 a 4 4 0 0 1 0,4 a 40 40 0 0 1 -20,14 a 8 8 0 0 1 -6,0 a 25 25 0 0 1 -8,-8 a 20 15 0 0 1 0,-8 l 3,-8 -22,-22 a 10 10 0 0 1 0,-8 z" />
  <path style="fill:#FFF;stroke:none;" d="m 92,120 a 10 10 0 0 1 8,0 l 20,10 -24,20 -24,-20 z" />
</svg>
EOF

# Make sure the file permissions are ok
chmod -R u+w,go+r-w,a-s .

# chrome-sandbox needs to be setuid root for Brave to run
chmod 4711 opt/brave.com/brave/chrome-sandbox

# package as sfs
( 
cd ../ 
mksquashfs pkg brave-browser-${VERSION}-${ARCH}${TAG}.sfs
mv brave-browser-${VERSION}-${ARCH}${TAG}.sfs /tmp

)

# cleanup
rm -r $REPACKDIR

# And ... we're done! ;)
echo 'brave was successfully packaged'
sleep 2
exit 0
